@@ -25,10 +25,4 @@ char * fromUtf8(char * to);
 
 void setLocaleCharset();
 
-int setCharSetConversion(char * to, char * from);
-
-char * convStrDup(char * string);
-
-void closeCharSetConversion();
-
 #endif
