@@ -117,9 +117,9 @@ int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 		arglens[i] = strlen(argv[i]);
 	}
 
-	/* get list of songs to add */
-	mpd_sendListallCommand(conn,"");
-	printErrorAndExit(conn);
+	 /* get list of songs to add */
+	 mpd_sendListallCommand(conn,"");
+	 printErrorAndExit(conn);
 
 	while((entity = mpd_getNextInfoEntity(conn))) {
 		if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
@@ -183,7 +183,6 @@ int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 
 int cmd_crop( int argc, char ** argv, mpd_Connection * conn )
 {
-
 	mpd_Status *status = getStatus( conn );
 	int length = ( status->playlistLength - 1 );
 
@@ -218,7 +217,6 @@ int cmd_crop( int argc, char ** argv, mpd_Connection * conn )
 		DIE( "You need to be playing to crop the playlist\n" );
 
 	}
-
 }
 
 int cmd_del ( int argc, char ** argv, mpd_Connection * conn )
@@ -245,6 +243,13 @@ int cmd_del ( int argc, char ** argv, mpd_Connection * conn )
 		else s = argv[i];
 
 		range[0] = strtol(s,&t,10);
+
+		/* If argument is 0 current song and we're not stopped */
+		if(range[0] == 0 && strlen(s) == 1 && \
+			(status->state == MPD_STATUS_STATE_PLAY ||
+			status->state == MPD_STATUS_STATE_PAUSE))
+			range[0] = status->song+1;
+
 		if(s==t)
 			DIE("error parsing song numbers from: %s\n",argv[i]);
 		else if(*t=='-') {
@@ -277,6 +282,7 @@ int cmd_del ( int argc, char ** argv, mpd_Connection * conn )
 	mpd_sendCommandListEnd(conn);
 	my_finishCommand(conn);
 
+	mpd_freeStatus(status);
 	free(songsToDel);
 	return 0;
 }
@@ -314,7 +320,7 @@ int cmd_outputs( int argc, char ** argv, mpd_Connection * conn )
 		}
 		else
 		{
-			DIE( "error: cannot receive the current outputs\n" );
+			DIE( "cannot receive the current outputs\n" );
 		}
 		mpd_freeOutputElement( output );
 	}
@@ -366,6 +372,13 @@ int cmd_play ( int argc, char ** argv, mpd_Connection * conn )
 			DIE("error parsing song numbers from: %s\n",argv[i]);
 
 		song--;
+
+		/* This is necessary, otherwise mpc will output the wrong playlist number */
+		mpd_Status * status = getStatus(conn);
+		i = status->playlistLength;
+		mpd_freeStatus(status);
+		if(song >= i)
+			DIE("song number greater than playlist length.\n");
 	}
 
 	mpd_sendPlayCommand(conn,song);
@@ -530,8 +543,14 @@ int cmd_move ( int argc, char ** argv, mpd_Connection * conn )
 int cmd_playlist ( int argc, char ** argv, mpd_Connection * conn )
 {
 	mpd_InfoEntity * entity;
+	mpd_Status * status;
 	int count = 0;
 
+	mpd_sendStatusCommand(conn);
+	printErrorAndExit(conn);
+	status = mpd_getStatus(conn);
+	printErrorAndExit(conn);
+	mpd_finishCommand(conn);
 	mpd_sendPlaylistInfoCommand(conn,-1);
 	printErrorAndExit(conn);
 
@@ -539,7 +558,7 @@ int cmd_playlist ( int argc, char ** argv, mpd_Connection * conn )
 		if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
 			mpd_Song * song = entity->info.song;
 
-			printf("#%i) ", 1+count);
+			printf("%s%i) ", (status->song == count)?">":" ", 1+count);
 			pretty_print_song(song);
 			printf("\n");
 
@@ -549,6 +568,7 @@ int cmd_playlist ( int argc, char ** argv, mpd_Connection * conn )
 	}
 
 	my_finishCommand(conn);
+	mpd_freeStatus(status);
 
 	return 0;
 }
@@ -730,54 +750,53 @@ int cmd_load ( int argc, char ** argv, mpd_Connection * conn )
 int cmd_search ( int argc, char ** argv, mpd_Connection * conn ) 
 {
 	mpd_InfoEntity * entity;
-	char * search;
-	int table = -1;
-	int i;
+	int type, i, numconstraints = 0;
+	struct _constraints {
+		int type;
+		char * query;
+	} * constraints;
 
-	struct search_types {
-		const char * field;
-		const int table;
-	} mpc_search [] = {
-		{"album", MPD_TABLE_ALBUM },
-		{"artist", MPD_TABLE_ARTIST },
-		{"title", MPD_TABLE_TITLE },
-		{"filename", MPD_TABLE_FILENAME },
-		{ NULL }
-	};
-
-	for(i=0;mpc_search[i].field;++i)
-		if (! strcmp(mpc_search[i].field,argv[0]))
-			table = mpc_search[i].table;
-	if (-1==table) {
-		fprintf(stderr,"\"%s\" is not one of: ", argv[0]);
-		for(i=0;mpc_search[i].field;++i)
-			fprintf(stderr,"%s%s%s",
-					( !mpc_search[i+1].field ? "or " : ""),
-					mpc_search[i].field,
-					(  mpc_search[i+1].field ? ", "   : "\n")
-			       );
-		return -1;
+	if (argc % 2 != 0) {
+		DIE("number of arguments must be a multiple of 2 "
+		    "(pairs of search types and queries)\n");
 	}
 
-	for(i=1; i<argc && (search = toUtf8(argv[i])); i++)  {
-		mpd_sendSearchCommand(conn,table,search);
-		printErrorAndExit(conn);
+	constraints = malloc(sizeof(struct _constraints)*argc/2);
 
-		while((entity = mpd_getNextInfoEntity(conn))) {
-			printErrorAndExit(conn);
-			if(entity->type==MPD_INFO_ENTITY_TYPE_DIRECTORY) {
-				mpd_Directory * dir = entity->info.directory;
-				printf("%s\n",fromUtf8(dir->path));
-			}
-			else if(entity->type==MPD_INFO_ENTITY_TYPE_SONG) {
-				mpd_Song * song = entity->info.song;
-				printf("%s\n",fromUtf8(song->file));
-			}
-			mpd_freeInfoEntity(entity);
+	for (i = 0; i < argc; i += 2) {
+		type = get_search_type(argv[i]);
+		if (type < 0) {
+			free(constraints);
+			return -1;
 		}
 
-		my_finishCommand(conn);
+		constraints[numconstraints].type = type;
+		constraints[numconstraints].query = argv[i+1];
+		numconstraints++;
 	}
+
+	mpd_startSearch(conn, 0);
+
+	for (i = 0; i < numconstraints; i++) {
+		mpd_addConstraintSearch(conn, constraints[i].type,
+		                        toUtf8(constraints[i].query));
+	}
+
+	free(constraints);
+	mpd_commitSearch(conn);
+	printErrorAndExit(conn);
+
+	while ((entity = mpd_getNextInfoEntity(conn))) {
+		printErrorAndExit(conn);
+		if (entity->type == MPD_INFO_ENTITY_TYPE_DIRECTORY)
+			printf("%s\n", fromUtf8(entity->info.directory->path));
+		else if (entity->type == MPD_INFO_ENTITY_TYPE_SONG)
+			printf("%s\n", fromUtf8(entity->info.song->file));
+		mpd_freeInfoEntity(entity);
+	}
+
+	my_finishCommand(conn);
+
 	return 0;
 }
 
@@ -885,7 +904,6 @@ int cmd_crossfade ( int argc, char ** argv, mpd_Connection * conn )
 		printErrorAndExit(conn);
 	}
 	return 0;
-
 }
 
 int cmd_version ( int argc, char ** argv, mpd_Connection * conn )
