@@ -40,6 +40,7 @@
 #include <sys/time.h>
 #include <stdarg.h>
 #define MPD_BUFFER_MAX_LENGTH	50000
+#define MPD_ERRORSTR_MAX_LENGTH	1000
 #define MPD_WELCOME_MESSAGE	"OK MPD "
 
 #define MPD_ERROR_TIMEOUT	10 /* timeout trying to talk to mpd */
@@ -88,8 +89,9 @@ typedef enum mpd_TagItems
 	MPD_TAG_ITEM_COMMENT,
 	MPD_TAG_ITEM_DISC,
 	MPD_TAG_ITEM_FILENAME,
+	MPD_TAG_ITEM_ANY,
 	MPD_TAG_NUM_OF_ITEM_TYPES
-}mpd_TagItems;
+} mpd_TagItems;
 
 extern char * mpdTagItemKeys[MPD_TAG_NUM_OF_ITEM_TYPES];
 
@@ -107,7 +109,7 @@ typedef struct _mpd_Connection {
 	/* use this to check the version of mpd */
 	int version[3];
 	/* IMPORTANT, you want to get the error messages from here */
-	char errorStr[MPD_BUFFER_MAX_LENGTH+1];
+	char errorStr[MPD_ERRORSTR_MAX_LENGTH+1];
 	int errorCode;
 	int errorAt;
 	/* this will be set to MPD_ERROR_* if there is an error, 0 if not */
@@ -262,6 +264,8 @@ typedef struct _mpd_Song {
 	char *genre;
 	/* Composer */
 	char *composer;
+	/* Performer */
+	char *performer;
 	/* Disc */
 	char *disc;
 	/* Comment */
@@ -414,10 +418,10 @@ void mpd_sendListallInfoCommand(mpd_Connection * connection, const char * dir);
 /* non-recursive version of ListallInfo */
 void mpd_sendLsInfoCommand(mpd_Connection * connection, const char * dir);
 
-#define MPD_TABLE_ARTIST	0
-#define MPD_TABLE_ALBUM		1
-#define MPD_TABLE_TITLE		2
-#define MPD_TABLE_FILENAME	3
+#define MPD_TABLE_ARTIST	MPD_TAG_ITEM_ARTIST
+#define MPD_TABLE_ALBUM		MPD_TAG_ITEM_ALBUM
+#define MPD_TABLE_TITLE		MPD_TAG_ITEM_TITLE
+#define MPD_TABLE_FILENAME	MPD_TAG_ITEM_FILENAME
 
 void mpd_sendSearchCommand(mpd_Connection * connection, int table,
 		const char * str);
@@ -434,7 +438,7 @@ char * mpd_getNextArtist(mpd_Connection * connection);
 
 char * mpd_getNextAlbum(mpd_Connection * connection);
 
-char * mpd_getNextTag(mpd_Connection *connection, int table);
+char * mpd_getNextTag(mpd_Connection *connection, int type);
 
 /* list artist or albums by artist, arg1 should be set to the artist if
  * listing albums by a artist, otherwise NULL for listing all artists or albums
@@ -446,6 +450,8 @@ void mpd_sendListCommand(mpd_Connection * connection, int table,
 
 void mpd_sendAddCommand(mpd_Connection * connection, const char * file);
 
+int mpd_sendAddIdCommand(mpd_Connection *connection, const char *file);
+
 void mpd_sendDeleteCommand(mpd_Connection * connection, int songNum);
 
 void mpd_sendDeleteIdCommand(mpd_Connection * connection, int songNum);
@@ -544,6 +550,7 @@ void mpd_freeOutputElement(mpd_OutputEntity * output);
  * Queries mpd for the allowed commands
  */
 void mpd_sendCommandsCommand(mpd_Connection * connection);
+
 /**
  * @param connection a #mpd_Connection
  *
@@ -568,6 +575,7 @@ char *mpd_getNextCommand(mpd_Connection *connection);
  *
  */
 void mpd_sendListPlaylistInfoCommand(mpd_Connection *connection, char *path);
+
 /**
  * @param connection a MpdConnection
  * @param path	the path to the playlist.
@@ -584,27 +592,29 @@ void mpd_sendListPlaylistCommand(mpd_Connection *connection, char *path);
  * starts a search, use mpd_addConstraintSearch to add
  * a constraint to the search, and mpd_commitSearch to do the actual search
  */
-void mpd_startSearch(mpd_Connection * connection,int exact);
+void mpd_startSearch(mpd_Connection *connection, int exact);
+
 /**
  * @param connection a #mpd_Connection
- * @param field
+ * @param type
  * @param name
- *
  */
-void mpd_addConstraintSearch(mpd_Connection *connection, int field, char *name);
+void mpd_addConstraintSearch(mpd_Connection *connection, int type, const char *name);
+
 /**
  * @param connection a #mpd_Connection
- *
  */
 void mpd_commitSearch(mpd_Connection *connection);
 
 /**
  * @param connection a #mpd_Connection
- * @param field The field to search
+ * @param type The type to search for
  *
  * starts a search for fields... f.e. get a list of artists would be:
+ * @code
  * mpd_startFieldSearch(connection, MPD_TAG_ITEM_ARTIST);
  * mpd_commitSearch(connection);
+ * @endcode
  *
  * or get a list of artist in genre "jazz" would be:
  * @code
@@ -614,10 +624,10 @@ void mpd_commitSearch(mpd_Connection *connection);
  * @endcode
  *
  * mpd_startSearch will return  a list of songs (and you need mpd_getNextInfoEntity)
- * this one will return a list of only one field (the field specified with field) and you need
+ * this one will return a list of only one field (the one specified with type) and you need
  * mpd_getNextTag to get the results
  */
-void mpd_startFieldSearch(mpd_Connection * connection,int field);
+void mpd_startFieldSearch(mpd_Connection *connection, int type);
 #ifdef __cplusplus
 }
 #endif
