@@ -35,59 +35,59 @@
 #include <sys/param.h>
 
 struct _mpc_table {
-	const char * command;
-	const int min, max;	/* min/max arguments allowed, -1 = unlimited */
-	int pipe;		/*
-				1: implicit pipe read, `-' optional as argv[2]
-				2: explicit pipe read `-' needed as argv[2]
-				3: explicit pipe read `-' needed as argv[3]
-				multipled by -1 if used, so that it can signal a free()
-				before the program exits
-				*/
+	const char *command;
+	const int min, max;   /* min/max arguments allowed, -1 = unlimited */
+	int pipe;             /**
+	                       * 1: implicit pipe read, `-' optional as argv[2]
+	                       * 2: explicit pipe read, `-' needed as argv[2]
+	                       *
+	                       * multipled by -1 if used, so that it can signal
+	                       * a free() before the program exits
+	                       */
 	cmdhandler handler;
-	const char * usage;
-	char * help;		/* NULL means they won't be shown in help */
+	const char *usage;
+	char *help;           /* NULL means they won't be shown in help */
 } mpc_table [] = {
-	/* Command Name, Minimum Arguments, Maximium args, pipe read (see above), Argument description, command description */
-	{"add", 0, -1, 1, cmd_add,"<file>","Add a song to the current playlist" },
-	{"crop", 0, 0, 0, cmd_crop,"","Remove all songs except for the currently playing song" },
-	{"del", 0, -1, 1, cmd_del,"<position>","Remove a song from the current playlist" },
-	{"play", 0, -1, 2, cmd_play,"[<position>]","Start playing at <position> (default: 1)" },
-	{"next", 0, 0, 0, cmd_next, "","Play the next song in the current playlist"},
-	{"prev", 0, 0, 0, cmd_prev,"","Play the previous song in the current playlist"},
-	{"pause", 0, 0, 0, cmd_pause, "", "Pauses the currently playing song"},
-	{"toggle", 0, 0, 0, cmd_toggle, "", "Toggles Play/Pause, plays if stopped"},
-	{"stop", 0, 0, 0, cmd_stop,"", "Stop the currently playing playlists"},
-	{"seek", 1, 1, 0, cmd_seek,"[+-][HH:MM:SS] or <0-100>%","Seeks to the specified position"},
-	{"clear", 0, 0, 0, cmd_clear,"", "Clear the current playlist"},
-	{"outputs", 0, 0, 0, cmd_outputs,"", "Show the current outputs"},
-	{"enable", 1, 1, 0, cmd_enable, "<output #>", "Enable a output"},
-	{"disable", 1, 1, 0, cmd_disable, "<output #>", "Disable a output"},
-	{"shuffle", 0, 0, 0, cmd_shuffle,"", "Shuffle the current playlist"},
-	{"move", 2, 2, 0, cmd_move,"<from> <to>", "Move song in playlist"},
-	{"playlist", 0, 0, 0, cmd_playlist, "", "Print the current playlist"},
-	{"listall", 0, -1, 2, cmd_listall,"[<file>]","List all songs in the music dir"},
-	{"ls", 0, -1, 2, cmd_ls,"[<directory>]","List the contents of <directory>"},
-	{"lsplaylists", 0, -1, 2, cmd_lsplaylists,"", "Lists currently available playlists"},
-	{"load", 0, -1, 1, cmd_load,"<file>","Load <file> as a playlist"},
-	{"save", 1, 1, 0, cmd_save,"<file>", "Saves a playlist as <file>"},
-	{"rm", 1, 1, 0, cmd_rm,"<file>","Removes a playlist"},
-	{"volume", 0, 1, 0, cmd_volume,"[+-]<num>","Sets volume to <num> or adjusts by [+-]<num>"},
-	{"repeat", 0, 1, 0, cmd_repeat,"<on|off>", "Toggle repeat mode, or specify state"},
-	{"random", 0, 1, 0, cmd_random,"<on|off>", "Toggle random mode, or specify state"},
-	{"search", 2, -1, 3, cmd_search, "<type> <queries>",	"Search for a song"},
-	{"crossfade", 0, 1, 0, cmd_crossfade,"[<seconds>]","Set and display crossfade settings"},
-	{"update", 0, -1, 2, cmd_update, "[<path>]", "Scans music directory for updates"},
-	{"stats", 0, -1, 0, cmd_stats, "", "Displays statistics about MPD"},
-	{"version", 0, 0, 0, cmd_version,"", "Reports version of MPD"},
+	/* command,     min, max, pipe, handler,         usage, help */
+	{"add",         0,   -1,  1,    cmd_add,         "<file>", "Add a song to the current playlist"},
+	{"crop",        0,   0,   0,    cmd_crop,        "", "Remove all but the currently playing song"},
+	{"del",         0,   -1,  1,    cmd_del,         "<position>", "Remove a song from the current playlist"},
+	{"play",        0,   -1,  2,    cmd_play,        "[<position>]", "Start playing at <position> (default: 1)"},
+	{"next",        0,   0,   0,    cmd_next,        "", "Play the next song in the current playlist"},
+	{"prev",        0,   0,   0,    cmd_prev,        "", "Play the previous song in the current playlist"},
+	{"pause",       0,   0,   0,    cmd_pause,       "", "Pauses the currently playing song"},
+	{"toggle",      0,   0,   0,    cmd_toggle,      "", "Toggles Play/Pause, plays if stopped"},
+	{"stop",        0,   0,   0,    cmd_stop,        "", "Stop the currently playing playlists"},
+	{"seek",        1,   1,   0,    cmd_seek,        "[+-][HH:MM:SS]|<0-100>%", "Seeks to the specified position"},
+	{"clear",       0,   0,   0,    cmd_clear,       "", "Clear the current playlist"},
+	{"outputs",     0,   0,   0,    cmd_outputs,     "", "Show the current outputs"},
+	{"enable",      1,   1,   0,    cmd_enable,      "<output #>", "Enable a output"},
+	{"disable",     1,   1,   0,    cmd_disable,     "<output #>", "Disable a output"},
+	{"shuffle",     0,   0,   0,    cmd_shuffle,     "", "Shuffle the current playlist"},
+	{"move",        2,   2,   0,    cmd_move,        "<from> <to>", "Move song in playlist"},
+	{"playlist",    0,   0,   0,    cmd_playlist,    "", "Print the current playlist"},
+	{"listall",     0,   -1,  2,    cmd_listall,     "[<file>]", "List all songs in the music dir"},
+	{"ls",          0,   -1,  2,    cmd_ls,          "[<directory>]", "List the contents of <directory>"},
+	{"lsplaylists", 0,   -1,  2,    cmd_lsplaylists, "", "Lists currently available playlists"},
+	{"load",        0,   -1,  1,    cmd_load,        "<file>", "Load <file> as a playlist"},
+	{"save",        1,   1,   0,    cmd_save,        "<file>", "Saves a playlist as <file>"},
+	{"rm",          1,   1,   0,    cmd_rm,          "<file>", "Removes a playlist"},
+	{"volume",      0,   1,   0,    cmd_volume,      "[+-]<num>", "Sets volume to <num> or adjusts by [+-]<num>"},
+	{"repeat",      0,   1,   0,    cmd_repeat,      "<on|off>", "Toggle repeat mode, or specify state"},
+	{"random",      0,   1,   0,    cmd_random,      "<on|off>", "Toggle random mode, or specify state"},
+	{"search",      2,   -1,  0,    cmd_search,      "<type> <query>", "Search for a song"},
+	{"crossfade",   0,   1,   0,    cmd_crossfade,   "[<seconds>]", "Set and display crossfade settings"},
+	{"update",      0,   -1,  2,    cmd_update,      "[<path>]", "Scans music directory for updates"},
+	{"stats",       0,   -1,  0,    cmd_stats,       "", "Displays statistics about MPD"},
+	{"version",     0,   0,   0,    cmd_version,     "", "Reports version of MPD"},
 	/* loadtab, lstab, and tab used for completion-scripting only */
-	{"loadtab",0, 1, 0, cmd_loadtab,"<directory>",},
-	{"lstab",0, 1, 0, cmd_lstab,"<directory>",},
-	{"tab",0, 1, 0, cmd_tab,"<path>",},
+	{"loadtab",     0,   1,   0,    cmd_loadtab,     "<directory>", NULL},
+	{"lstab",       0,   1,   0,    cmd_lstab,       "<directory>", NULL},
+	{"tab",         0,   1,   0,    cmd_tab,         "<path>", NULL},
 	/* status was added for pedantic reasons */
-	{"status", 0, -1, 0, cmd_status, "", NULL },
+	{"status",      0,   -1,  0,    cmd_status,      "", NULL},
 	/* don't remove this, when mpc_table[i].command is NULL it will terminate the loop */
-	{ NULL }
+	{NULL}
 };
 
 static int print_help(char * progname, char * command)
@@ -197,7 +197,6 @@ static char ** check_args(int idx, int * argc, char ** argv)
 	if ( ( mpc_table[idx].pipe==1 &&
 		(2==*argc || (3==*argc && 0==strcmp(argv[2],STDIN_SYMBOL) )))
 	|| (mpc_table[idx].pipe==2 && (3==*argc && 0==strcmp(argv[2],STDIN_SYMBOL)))
-	|| (mpc_table[idx].pipe==3 && (4==*argc && 0==strcmp(argv[3],STDIN_SYMBOL)))
 	){
 		*argc = stdinToArgArray(&array);
 		mpc_table[idx].pipe *= -1;
