@@ -21,8 +21,9 @@
 
 #include "../config.h"
 
+#include "mpc_compat.h"
 #include "libmpdclient.h"
-#define STRING_LENGTH	(2*MAXPATHLEN)
+#define STRING_LENGTH	(8192)
 
 #define STDIN_SYMBOL	"-"
 
